#
# shellscript, generate gnuplot 3d-files(splot) from daten.def
#  by Joerg.Schulenburg
#
# use:
#  sh def2plot3d.sh daten.def >a.gpl
#  sh def2plot3d.sh -plt              # => a.ppm a.eps (obsolete?)
#
# ChangeLog:
#  ---TODO---   need R= in daten.def first, from m_lattice.c etc.
#               else print only bonds<=min_len_bond (overhead? colors?)
#  v1.0 Sep19 - 3d-def version based on def2pov(povray) script Aug01
#               for simpler usage of gnuplot and interactive viewing 3D
#               see also models/m_lattice.c out_splot()
#
# ToDo:
#  - daten.def should have R-vectors for bonds (good for TBC too)
#  - simpler is viewing using gnuplot splot (3D-plot) 2019-09
#
if test "$1" == "-plt"; then
 echo "gnuplot a.gpl"
 nice -30 gnuplot a.gpl
 # ppmtopgm a.ppm | 
 exit
fi
LANG=C    # printf %f of awk is language dependent, we want the x.y-format
LC_ALL=C  # 2019-09
#
# see /usr/doc/info/gawk.info.gz
# the daten.def file should have additional lines for better drawing:
# #===base=== b1x b1y b2x b2y        - base vektors b1 b2 default=( 1.0 0.0 0.0 1.0 )
#
# if test "$1" == "-pov"; then option="pov"; shift; else option=""; fi
if test -z "$1"; then file=daten.def
                 else file=$1
fi
if test ! -r $file; then echo "file $file not found";exit; fi
#
cat <<END
# gnuplot output - gnuplot version 4.6 (2013)
# output from def2plt3d.sh
# set view rot_x,rot_z,scale,scale_z # screen=(x,y)
# set view  0,0  # from z to xy l6=checker l7=J1-J2-square
# set view 56,0  # from z+y     l6=kago    l7=triangular
set xlabel "x" 
set ylabel "y" 
set zlabel "z"
END
awk -v zoom=10.0 '\
  function outs( x, y, z, s, c){ # number  c=color s=number
    printf "object { Site scale 1.0 translate <%d,%d,%d> }\n",x,y,z;
  }
  BEGIN { count=0;nr=0;sc=zoom; ox=0;oy=0;oz=0; rx=0;ry=0;rz=0;
    for( i=0; i<3*3; i++  ) { b[i]=0; e[i]=0; t[i]=0; }
    for( i=0; i<3*3; i+=4 ) { b[i]=1; e[i]=8; t[i]=8; }
    d=2;
  }
  !/^\#/ { count++; nr=count; }
  /^\#/  { nr=0 ; }
  /^\#===base===/  {
    if ( NF<10 ) { d=2 } else { d=3 }
    for( i=0; i<d; i++ ) for( j=0; j<d; j++ ) b[i*3+j]=$(2+i*d+j);
    printf "#DBG: base %g %g %g  %g %g %g  %g %g %g\n",\
       b[0],b[1],b[2],b[3],b[4],b[5],b[6],b[7],b[8];
  }
  /^\#===edge===/  {
    if ( NF<10 ) { d=2 } else { d=3 }
    for( i=0; i<d; i++ ) for( j=0; j<d; j++ ) t[i*3+j]=$(2+i*d+j);
    for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) { 
      e[i*3+j]=0;
      for( k=0; k<3; k++ ) {
        e[i*3+j]+=b[k*3+j]*t[i*3+k];
      }
    }
  }
  (nr == 1) { vn=$1;vnw=$2; printf "# NN=%d Nw=%d\n",vn,vnw }
  # particle-line: num rx ry rz [e#] [u#]
  (nr  > 1) && (nr < vn+2) {\
    if($2<ox) ox=$2; if($2>rx) rx=$2;
    if($3<oy) oy=$3; if($3>ry) ry=$3;
    if($4<oz) oz=$4; if($4>rz) rz=$4;
    sx[$1]=$2; sy[$1]=$3; sz[$1]=$4;  # store site position sx,sy,sz
  }
  # bond-line: num s1 s2 [R= # # #] [t#] [j#] ...
  (nr > vn+1 && nr < vnw + vn+2) {  # bonds = 2-site-terms
    i=$1        # numbering?
    w1[i]=$2;  # site1
    w2[i]=$3;  # site2
    wj[i]=1;
    if (match($0,"j2")) wj[i]=2; # bond-type for coloring
    if (match($0,"j3")) wj[i]=3; # bond-type for coloring
    if (match($0,"j4")) wj[i]=4; # bond-type for coloring
    if ($4 == "R=") { rrx[i]=$5; rry[i]=$6; rrz[i]=$7; rrr[i]=1;
      printf("#DBG %2d %2d - %2d R= %5s %5s %5s\n",\
        i,w1[i],w2[i],rrx[i],rry[i],rrz[i]);
      if (rrx[i]==0 && rry[i]==0 && rrz[i]==0) printf("#WARN R[%d]=0\n",i);
      } # bond-vec
    if (match($0,"+")) { wb[i]=1; } else { wb[i]=0; } # PBC
    if ( (sx[$2]-sx[$3])^2 \
       + (sy[$2]-sy[$3])^2 \
       + (sz[$2]-sz[$3])^2 > 4.1 ) { wb[i]=1; }
  }
  END {
    for ( i=0; i<vn; i++ ) { # sites.pos sx,sy,sz * base = sites.gpos
      ax[i]=(b[0]*sx[i] +b[3]*sy[i] +b[6]*sz[i] -ox)*sc; # svec[i]
      ay[i]=(b[1]*sx[i] +b[4]*sy[i] +b[7]*sz[i] -oy)*sc;
      az[i]=(b[2]*sx[i] +b[5]*sy[i] +b[8]*sz[i] -oz)*sc;
      # c=color=0 i=number
      printf "set label %3d \"%d\" at %7.3f,%7.3f,%7.3f # shift by 1,1,1\n",\
        i+1, i, sc*(ax[i]+0.1), sc*(ay[i]+0.1), sc*(az[i]+0.1)
    }  # for vn
    for ( i=0; i<vnw; i++ ) {
      arx[i]=(b[0]*rrx[i] +b[3]*rry[i] +b[6]*rrz[i])*sc;  # Rvec[i]
      ary[i]=(b[1]*rrx[i] +b[4]*rry[i] +b[7]*rrz[i])*sc;
      arz[i]=(b[2]*rrx[i] +b[5]*rry[i] +b[8]*rrz[i])*sc;
      if (arx[i]==0 && ary[i]==0 && arz[i]==0) printf("#WARN R[%d]=0\n",i);
    } # for vnw
    maxcolor=0
    for ( i=0; i<vnw; i++ ) if (wj[i]>maxcolor) maxcolor= wj[i];
    printf "# maxcolor=maxJ=%d rrr[0]=%d(R=...)\n", maxcolor, rrr[0];

    printf "splot \"-\" u 1:2:3 t \"UC\" w l lw 1 lc 7,\\\n"
    for (j=0;j<=maxcolor;j++){ # bounds in blocks of same color
      printf " \"-\" u 1:2:3 t \"J%d\" with lines lw %d lc %d",\
         j+1,((j)?1:3),j+1  # followed by ",\\" if multiple colored bounds
      if (j<=maxcolor-1) { printf ",\\" }
      printf "\n"
    }
    # UC-output added 2019-03-10 2D-UC + UC.e3 ...
    #   shifted by (0,0,1)*0.05 to move outside of potential bonds
    printf("# UC e1 e2 e3 shifted by (0,0,1)*0.05*zoom\n" \
           "% 8.4f % 8.4f % 8.4f  # origin 0,0,0\n" \
           "% 8.4f % 8.4f % 8.4f  # uc.e1\n" \
           "% 8.4f % 8.4f % 8.4f  # uc.e1+e2\n" \
           "% 8.4f % 8.4f % 8.4f  # uc.e2\n" \
           "% 8.4f % 8.4f % 8.4f  # origin 0,0,0\n" \
           "% 8.4f % 8.4f % 8.4f  # uc.e3\ne\n",
           0.0, 0.0, 0.05*sc,   
           e[0*3+0]*sc, e[0*3+1]*sc, (e[0*3+2]+0.05)*sc, # /* uc.e1 */
           (e[0*3+0]+e[1*3+0])*sc,
            (e[0*3+1]+e[1*3+1])*sc,
             (e[0*3+2]+e[1*3+2]+0.05)*sc,  # uc.e1+e2 
           e[1*3+0]*sc, e[1*3+1]*sc, (e[1*3+2]+0.05)*sc, # /* uc.e2 */
           0.0, 0.0, 0.05*sc,   
           e[2*3+0]*sc, e[2*3+1]*sc, (e[2*3+2]+0.05)*sc); # /* uc.e3 */
    for (j=0;j<=maxcolor;j++){ # bounds in blocks of same color 
      # TODO plot half bonds on both sites, fit together or not at UC border
      printf "# bonds j%d color=%d\n",j+1,j;
      for ( i=0; i<vnw; i++ ) {
        if (wj[i]!=j+1 || (wb[i]!=0 && rrr[i]==0)) continue;
        i1=w1[i];
        i2=w2[i];
        # plotting needs bound directions (not in old pre_v2019-09 daten.def)
        r1[0]=ax[i1]; r1[1]=ay[i1]; r1[2]=az[i1];
        r2[0]=ax[i2]; r2[1]=ay[i2]; r2[2]=az[i2];
        r3[0]=ax[i2]; r3[1]=ay[i2]; r3[2]=az[i2];
        r4[0]=ax[i1]; r4[1]=ay[i1]; r4[2]=az[i1];
        # printf "# bond %2d  %2d - %2d\n",i,i1,i2
        if (arx[i]==0 && ary[i]==0 && arz[i]==0) printf("#WARN R[%d]=0\n",i);
        if ( rrr[i]!=0 ) {  # new def-format using R= ...
          r2[0]= ax[i1] +0.5*arx[i];
          r2[1]= ay[i1] +0.5*ary[i];
          r2[2]= az[i1] +0.5*arz[i];
          r4[0]= ax[i2] -0.5*arx[i];
          r4[1]= ay[i2] -0.5*ary[i];
          r4[2]= az[i2] -0.5*arz[i];
        }
        # TODO need R= ... in daten.def or getting bad figure for borders
        printf "% 7.3f % 7.3f % 7.3f\n% 7.3f % 7.3f % 7.3f " \
               "# b%d:  s%d-s%d J%d\n\n\n",
                  r1[0], r1[1], r1[2],
                  r2[0], r2[1], r2[2],
                  i, i1, i2, wj[i];
        if (rrr[i]==1)
        printf "% 7.3f % 7.3f % 7.3f\n% 7.3f % 7.3f % 7.3f " \
               "# invers-direction\n\n\n",
                  r3[0], r3[1], r3[2],
                  r4[0], r4[1], r4[2];

      } # for i<vnw
      printf "e\n"
    } # for j<maxcolor
    printf "pause -1\n"
  } # END
  ' $file
exit


